var SelectRna = function()
{
	var selectrna = {};
	
	var EXTERNAL_RNA_NAME = 'External.r';
	var debug = 0;
	
	var graphics;
	var pen;
	
	var quantitiesAreValid = false;
	var subtypeRna;

	// Page initialization
	selectrna.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', selectrna.validateStep1);

		Events.addEventHandler('pool_schema', 'change', selectrna.poolSchemaOnChange);
		Events.addEventHandler('quantity_regular', 'change', selectrna.quantityOnChange);
		Events.addEventHandler('quantity_qc', 'change', selectrna.quantityOnChange);
		Events.addEventHandler('quantity_regular', 'keypress', Events.numberOnly);
		Events.addEventHandler('quantity_qc', 'keypress', Events.numberOnly);
		
		Events.addEventHandler('warning_quantity', 'change', selectrna.warningLevelOnChange);
		Events.addEventHandler('warning_quality_score', 'change', selectrna.warningLevelOnChange);
		Events.addEventHandler('warning_quantity', 'keypress', Events.numberOnly);
		Events.addEventHandler('warning_quality_score', 'keypress', Events.numberOnly);
		
		Events.addEventHandler('location', 'click', selectrna.toggleInfo);
		Events.addEventHandler('quantity', 'click', selectrna.toggleInfo);
		Events.addEventHandler('quality-score', 'click', selectrna.toggleInfo);
		Events.addEventHandler('ndconc', 'click', selectrna.toggleInfo);
		Events.addEventHandler('volumes', 'click', selectrna.toggleInfo);
		Events.addEventHandler('qiacube-date', 'click', selectrna.toggleInfo);
		
		Events.addEventHandler('pool-row', 'click', selectrna.togglePool);
		Events.addEventHandler('pool-row', 'mouseover', selectrna.highlightPool);
		Events.addEventHandler('pool-row', 'mouseout', selectrna.highlightPool);
		
		Buttons.addClickHandler('btnLoadFromFile', selectrna.selectFile);
		Buttons.addClickHandler('btnAutoSelect', selectrna.autoSelect);
		Buttons.addClickHandler('btnManualSelect', selectrna.manualSelect);
		Events.addEventHandler('plate', 'base-selected', selectrna.manuallySelected);
		Buttons.addClickHandler('btnStratagene', selectrna.setToStratagene);
		Buttons.addClickHandler('btnExternal', selectrna.setToExternal);
		Buttons.addClickHandler('btnToggleQc', selectrna.toggleQc);
		Buttons.addClickHandler('btnComment', selectrna.commentSelected);
		Buttons.addClickHandler('btnCutSelected', selectrna.cutSelected);
		Buttons.addClickHandler('btnCopySelected', selectrna.copySelected);
		Buttons.addClickHandler('btnPasteToSelected', selectrna.pasteToSelected);
		Buttons.addClickHandler('btnSwitch', selectrna.switchSelected);
		
		Events.addEventHandler('iconSpecialSelect', 'click', selectrna.toggleSpecialSelect);
		Events.addEventHandler('plate', 'mouseup', selectrna.contextEvent);
		Events.addEventHandler('plate', 'contextmenu', selectrna.contextEvent);
		Events.addEventHandler('mnuComment', 'click', selectrna.commentSelected);
		Events.addEventHandler('mnuToggleQc', 'click', selectrna.toggleQc);
		Events.addEventHandler('mnuCutSelected', 'click', selectrna.cutSelected);
		Events.addEventHandler('mnuCopySelected', 'click', selectrna.copySelected);
		Events.addEventHandler('mnuPasteToSelected', 'click', selectrna.pasteToSelected);
		Events.addEventHandler('mnuSwitch', 'click', selectrna.switchSelected);
		Events.addEventHandler('mnuCaseSummary', 'click', selectrna.showCaseSummary);
		Events.addEventHandler('mnuFlagLowQualityScore', 'click', selectrna.flagSelected);
		Events.addEventHandler('mnuFlagNotEnoughRemainingQuantity', 'click', selectrna.flagSelected);
		Events.addEventHandler('mnuFlagOther', 'click', selectrna.flagSelected);

		Events.addEventHandler('flaggedRna', 'click', selectrna.showFlaggedRna);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', selectrna.submit);
		
		graphics = new jsGraphics(Doc.element('canvas'));
		pen = new jsPen(new jsColor('#2288AA'), 2);
		
		var url = '../MRna.servlet?ID='+App.getSessionId();
		url += '&cmd=GetStratagene';
		Wizard.asyncJsonRequest(url, selectrna.stratageneLoaded);

		var url = '../MRna.servlet?ID='+App.getSessionId();
		url += '&cmd=GetNextAutoGeneratedPlateName&bioPlateType=MRNA';
		Wizard.showLoadingAnimation('Loading next mRNA plate');
		Wizard.asyncJsonRequest(url, selectrna.nextPlateNameLoaded);
	}

	selectrna.stratageneLoaded = function(response)
	{
		var frm = document.forms['reggie'];
		var stratagene = response.stratagene;

		for (var i = 0; i < stratagene.length; i++)
		{
			var s = stratagene[i];
			var name = s.name;
			if (s.bioWell)
			{
				name += ' -- ' + s.bioWell.bioPlate.name + ' ('+s.bioWell.location + ')';
			}
			frm.stratagene[frm.stratagene.length] = new Option(name, s.name);
		}
	}

	selectrna.nextPlateNameLoaded = function(response)
	{
		var frm = document.forms['reggie'];

		var schema = PoolSchema.initList(frm.pool_schema);
		Plate.init(8, 12, schema, WellPainter);
		Plate.paint(Plate.getWells());
		PoolSchema.buildPoolTableRow(schema, 12, true);
		
		Plate.name = response.name;
		Doc.element('plateName').innerHTML = Strings.encodeTags(Plate.name);
	
		Events.sendChangeEvent('quantity_regular');
		Events.sendChangeEvent('warning_quantity');
		
		Doc.show('step-1');
		Doc.show('gocancel');
		Doc.show('goregister');
		Wizard.setNoConfirmOnFirstStep(false);
		Wizard.keepSessionAlive();
	}

	
	selectrna.quantityOnChange = function(event)
	{
		var frm = document.forms['reggie'];
		quantitiesAreValid = false;
		
		var qRegular = parseFloat(frm.quantity_regular.value);
		if (!(qRegular > 0))
		{
			Wizard.setInputStatus('quantities', 'invalid', 'Amount must be ≥ 0.');
			return;
		}
		var qQc = parseFloat(frm.quantity_qc.value);
		if (!(qQc > qRegular))
		{
			Wizard.setInputStatus('quantities', 'invalid', 'Amount QC must be &gt; ' + qRegular + '.');
			return;
		}
		
		quantitiesAreValid = true;
		QUANTITY_REGULAR = qRegular;
		QUANTITY_QC = qQc;
		
		Wizard.setInputStatus('quantities', 'valid');
		if (LOW_QUANTITY_WARNING_LIMIT && QUALITY_SCORE_WARNING_LIMIT)
		{
			Plate.paint(Plate.getWells());
		}
	}

	selectrna.warningLevelOnChange = function(event)
	{
		var frm = document.forms['reggie'];

		var wQuantity = parseFloat(frm.warning_quantity.value);
		if (!(wQuantity > 0))
		{
			Wizard.setInputStatus('warnings', 'invalid', 'Remaining quantity must be ≥ 0.');
			return;
		}
		var wQualityScore = parseFloat(frm.warning_quality_score.value);
		if (!(wQualityScore > 0))
		{
			Wizard.setInputStatus('warnings', 'invalid', 'RQS/RIN must be ≥ 0.');
			return;
		}
		
		LOW_QUANTITY_WARNING_LIMIT = wQuantity;
		QUALITY_SCORE_WARNING_LIMIT = wQualityScore;
		
		Wizard.setInputStatus('warnings', 'valid');
		Plate.paint(Plate.getWells());
	}
	
	selectrna.poolSchemaOnChange = function()
	{
		var frm = document.forms['reggie'];
		var schema = POOL_SCHEMA[frm.pool_schema.selectedIndex];
		Plate.setPoolSchema(schema);
		Plate.paint(Plate.getWells());
		PoolSchema.buildPoolTableRow(schema, Plate.columns, true);
	}
	
	selectrna.toggleInfo = function(event)
	{
		var frm = document.forms['reggie'];
		var show = event.currentTarget.checked;
		Doc.addOrRemoveClass('plate', 'hide-'+event.currentTarget.id, !show);
	}

	/**
		Open a popup dialog for selecting a data files.
	*/
	selectrna.selectFile = function()
	{
		Dialogs.openPopup('select_file.jsp?ID='+App.getSessionId(), 'SelectFile', 600, 400);	
	}

	/**
		Parse a tab-separated data file. The columns must be in the following order:
		0: RNA name
		1: Row letter (A-H)
		2: Column number (1-12)
		3: QC flag (can be empty)
	*/
	selectrna.parseRnaFile = function(data)
	{
		var lines = data.split(/[\n\r]+/);
	
		// Load information about all RNA items in a batch (will improve performance)
		var names = [];
		for (var i = 0; i < lines.length; i++)
		{
			var line = lines[i];
			if (line)
			{
				var cols = lines[i].split(/\t/);
				if (cols.length < 3) throw 'On line '+(i+1)+': Too few columns (' + cols.length + ')';
				names[names.length] = cols[0];
			}
		}
		Rna.loadInfoByNames(names);
		
		// Place RNA on the plate
		var duplicates = [];
		for (var i = 0; i < lines.length; i++)
		{
			var line = lines[i];
			if (line)
			{
				var cols = lines[i].split(/\t/);
	
				var rna = Rna.createByName(cols[0]);
				var row =  Reggie.alphaToWell(cols[1].toUpperCase());
				var col =  parseInt(cols[2], 10)-1;
				
				var well = Plate.getWell(row, col);
				if (!well) throw 'On line '+(i+1)+': Invalid plate coordinate ['+cols[1]+','+cols[2]+']';
	
				rna.qc = cols.length >= 4 && cols[3];
				Rna.unflag(rna);
				
				// Check for duplicate RNA on same position (which is an error)
				var pos = 'c'+col+'r'+row;
				if (duplicates[pos])
				{
					well.addDuplicate(rna);
				}
				else
				{
					duplicates[pos] = rna.name;
					well.setExtract(rna);
				}
			}
		}
		
		selectrna.updateNumFlaggedRna();
		
		// Check for replicates on the whole plate and repaint it
		Plate.checkReplicates(null, true);
		Plate.paint(Plate.getWells());
	}

	
	
	var currentSelected;
	var names = [];
	/**
		Open a popup dialog for manual selection of RNA.
	*/
	selectrna.manualSelect = function(event)
	{
		currentSelected = Plate.getSelected();
		if (currentSelected.length == 0)
		{
			Forms.showNotification(event.currentTarget, 'Please select one or more wells were RNA should be placed.');
			return;
		}
		
		if (subtypeRna == null) subtypeRna = Reggie.getSubtypeInfo('RNA');
	
		var url = '&resetTemporary=1';
		url += '&tmpfilter:INT:itemSubtype='+subtypeRna.id;
		url += '&tmpfilter:DATE:creationEvent.eventDate='+encodeURIComponent('<>');
		url += '&tmpfilter:FLOAT:remainingQuantity='+encodeURIComponent('>=1');
		url += '&'+encodeURIComponent('tmpfilter:STRING:&childCreationEvents(event.bioMaterial.name)')+'='+encodeURIComponent('<>%.m');

		Dialogs.selectItem('EXTRACT', 'plate', 1, url);
	}

	/**
		Callback method for manual selection.
	*/
	selectrna.manuallySelected = function(event)
	{
		names[names.length] = event.detail.name;
		
		if (event.detail.remaining == 0 && names.length > 0)
		{
			Rna.loadInfoByNames(names);
			
			for (var i = 0; i < names.length && i < currentSelected.length; i++)
			{
				var well = currentSelected[i];
				var name = names[i];
				
				// Create a new RNA object
				var rna = Rna.createByName(name);
				Rna.unflag(rna);
				well.setExtract(rna);
				well.selected = false;
			}
			
			Plate.checkReplicates();
			Plate.paint(currentSelected);
			selectrna.updateNumFlaggedRna();
			names = [];
		}
	}

	
	/**
		Let the wizard automatically select among unprocessed RNA items.
	*/	
	selectrna.autoSelect = function(event)
	{
		var frm = document.forms['reggie'];
		var wells = Plate.getWells();
		var selected = [];
		var ignore = [];
		var notEmpty = 0;
		
		// Count selected and non-empty wells and keep track of RNA that is already present
		for (var i = 0; i < wells.length; i++)
		{
			var well = wells[i];
			if (well.selected) 
			{
				selected[selected.length] = well;
				if (well.extract && !well.hasError()) notEmpty++;
			}
			if (well.extract && well.extract.id)
			{
				ignore[ignore.length] = well.extract.id;
			}
		}
		// Ignore RNA already flagged
		var flagged = Rna.getFlagged();
		for (var i = 0; i < flagged.length; i++)
		{
			var rna = flagged[i];
			if (rna.id) ignore[ignore.length] = rna.id;
		}
		
		if (selected.length == 0)
		{
			Forms.showNotification(event.currentTarget, 'Please select one or more wells were RNA should be placed.');
			return;
		}
			
		if (notEmpty > 0)
		{
			if (!confirm('Replace RNA in ' + notEmpty + ' wells?'))
			{
				return;
			}
		}
			
		var url = 'auto_select_rna.jsp?ID='+App.getSessionId();
		url += '&numToSelect='+selected.length;
		url += '&quantity='+encodeURIComponent(frm.quantity_regular.value);
		url += '&ignore='+ignore.join(',');
		Dialogs.openPopup(url, 'AutoSelectRNA', 750, 500);
	}
	
	selectrna.rnaSelected = function(response)
	{
		// Cache RNA information
		Rna.cacheInfo(response.rna);
		Rna.cacheInfo(response.flagged);
		
		selectrna.updateNumFlaggedRna();
		var selected = Plate.getSelected();
		
		for (var i = 0; i < response.rna.length; i++)
		{
			if (i < selected.length)
			{
				var well = selected[i];
				// Create a new RNA object
				var rna = Rna.createByName(response.rna[i].name);
				well.setExtract(rna);
				well.selected = false;
			}
		}
		
		// Check for replicates on the whole plate and repaint it
		Plate.checkReplicates(null, true);
		Plate.paint(Plate.getWells());
	}
	
	// Update the 'Flagged RNA' information
	selectrna.updateNumFlaggedRna = function()
	{
		var numFlagged = Rna.getFlagged().length;
		if (numFlagged > 0)
		{
			Doc.element('flaggedRna').innerHTML = numFlagged + ' flagged RNA items.';
			Doc.show('flaggedRna');
		}
		else
		{
			Doc.hide('flaggedRna');
		}
	}

	/**
		Add 'Stratagene' to the selected wells.
	*/
	selectrna.setToStratagene = function(event)
	{
		var frm = document.forms['reggie'];
		selectrna.setToSpecial(event, frm.stratagene.value, true);
	}


	/**
		Add 'External RNA' to the selected wells.
	*/
	selectrna.setToExternal = function(event)
	{
		selectrna.setToSpecial(event, EXTERNAL_RNA_NAME, false);	
	}
	
	/**
		Add special RNA to the selected wells.
	*/
	selectrna.setToSpecial = function(event, specialName, qc)
	{
		var wells = Plate.getSelected();
		
		if (wells.length == 0)
		{
			Forms.showNotification(event.currentTarget, 'No wells have been selected');
			return;
		}
		
		// Count non-empty wells
		var count = 0;
		for (var i = 0; i < wells.length; i++)
		{
			var well = wells[i];
			if (well.extract && !well.hasError()) count++;
		}
		
		if (count > 0)
		{
			if (!confirm('Replace RNA in ' + count + ' wells with '+specialName+'?'))
			{
				return;
			}
		}
		
		var info = Rna.infoByName(specialName);
		if (!info.id)
		{
			alert('Could not find any RNA with name=\''+specialName+'\'. Please check that it exists on the server.');
			return;
		}
		
		for (var i = 0; i < wells.length; i++)
		{
			var well = wells[i];
			var rna = Rna.createByInfo(info);
			rna.qc = qc;
			well.setExtract(rna);
			well.selected = false;
		}
		
		Plate.paint(wells);
		
		// Check for replicated RNA if some have been replaced with Stratagene
		if (count > 0) Plate.checkReplicates();
	}
	
	/**
		Toggle the QC flag for the selected wells. The first non-empty
		well is toggled and the rest of the wells will use the same new QC
		value.
	*/
	selectrna.toggleQc = function(event)
	{
		var wells = Plate.getSelected();
	
		if (wells.length == 0) 
		{
			Forms.showNotification(event.currentTarget, 'No wells have been selected');
			return;
		}
		
		var gotQc = false;
		var newQc;
		var count = 0;
		for (var i = 0; i < wells.length; i++)
		{
			var well = wells[i];
			if (well.extract)
			{
				// Toggle QC flag for the first well with RNA, then use the same flag for the rest
				if (!gotQc)
				{
					gotQc = true;
					newQc = !well.extract.qc;
				}
				well.extract.qc = newQc;
				count++;
			}
		}
		
		if (count == 0)
		{
			Forms.showNotification(event.currentTarget, 'None of the selected wells contain any RNA');
		}
		else
		{
			Plate.paint(wells);
		}
	}

	// Set a comment on the selected wells
	selectrna.commentSelected = function(event)
	{
		var wells = Plate.getSelected();
		
		if (wells.length == 0)
		{
			Forms.showNotification(event.currentTarget, 'No wells have been selected');
			return;
		}
		
		var count = 0;
		var comment = '';
		for (var i = 0; i < wells.length; i++)
		{
			var well = wells[i];
			if (well.extract) 
			{
				count++;
				if (well.extract.comment) comment = well.extract.comment;
			}
		}
			
		if (count == 0)
		{
			Forms.showNotification(event.currentTarget, 'None of the selected wells contain any RNA');
			return;
		}

		comment = prompt('Comment', comment || '');
		if (comment == null) return;
		
		if (comment == '') comment = null;
		for (var i = 0; i < wells.length; i++)
		{
			var well = wells[i];
			if (well.extract) well.extract.comment = comment;
		}
		Plate.paint(wells);
	}

	/**
		Empty the selected wells from RNA. They can later be pasted again.
	*/
	selectrna.cutSelected = function(event)
	{
		var wells = Plate.getSelected();
		
		if (wells.length == 0)
		{
			Forms.showNotification(event.currentTarget, 'No wells have been selected');
			return;
		}
		
		var count = 0;
		var valid = 0;
		for (var i = 0; i < wells.length; i++)
		{
			var well = wells[i];
			if (well.extract) 
			{
				count++;
				if (!well.hasError()) valid++;
			}
		}
		
		if (count == 0)
		{
			Forms.showNotification(event.currentTarget, 'None of the selected wells contain any RNA');
			return;
		}
		
		// Ask for confirmation before deleting from valid wells
		if (valid > 0)
		{
			if (!confirm('Clear RNA from ' + count + ' of ' + wells.length + ' selected wells?'))
			{
				return;
			}
		}
	
		selectrna.copySelected();
		for (var i = 0; i < wells.length; i++)
		{
			var well = wells[i];
			well.setExtract(null);
			well.selected = false;
		}
		Plate.paint(wells);
		Plate.checkReplicates();
	}
	
	var copy;
	/**
		Copy information about the selected wells. 
	*/
	selectrna.copySelected = function(event)
	{
		// Clear existing wells in the copy
		var repaint = [];
		if (copy && copy.length > 0)
		{
			for (var i = 0; i < copy.length; i++)
			{
				var cp = copy[i];
				cp.well.doneWithCopy();
				repaint[repaint.length] = cp.well;
			}
		}
		Plate.paint(repaint);
		
		// Place selected wells in the copy
		var wells = Plate.getSelected();
		copy = [];
		for (var i = 0; i < wells.length; i++)
		{
			var well = wells[i];
			copy[copy.length] = well.makeCopy();
			well.selected = false;
		}
		Plate.paint(wells);
	}
	
	/**
		Paste information into the selected wells.
	*/
	selectrna.pasteToSelected = function(event)
	{
		if (!copy || copy.length == 0) 
		{
			Forms.showNotification(event.currentTarget, 'Nothing to paste. Please cut or copy wells first.');
			return;
		}
		
		var wells = Plate.getSelected();
		// Count non-empty and valid wells
		var count = 0;
		for (var i = 0; i < wells.length; i++)
		{
			var well = wells[i];
			if (well.extract && !well.hasError()) count++;
		}
		
		if (count > 0)
		{
			if (!confirm('Replace RNA in ' + count + ' wells with copy?'))
			{
				return;
			}
		}
		
		var wi = 0;
		var ci = 0;
		var copyEmpty;
		var askIfEmpty = true;
		var repaint = [];
		
		while (wi < wells.length && ci < copy.length)
		{
			var well = wells[wi];
			var cp = copy[ci];
			var rna = null;
			cp.well.doneWithCopy();
			repaint[repaint.length] = cp.well;
			if (cp.name)
			{
				rna = Rna.createByName(cp.name);
				rna.comment = cp.comment;
			}
			else
			{
				// The copy is from an empty well
				if (askIfEmpty)
				{
					askIfEmpty = false;
					copyEmpty = confirm('Do you want to copy empty wells? If not, only non-empty well are copied.');
				}
			}
			if (rna != null || copyEmpty)
			{
				well.setExtract(rna);
				well.selected = false;
				repaint[repaint.length] = well;
				wi++;
			}
			ci++;
		}
		Plate.paint(repaint);
		copy.splice(0, ci);
		Plate.checkReplicates();
	}

	/*
		If exactly two wells have been selected, switch the RNA in them.
	*/
	selectrna.switchSelected = function(event)
	{
		var wells = Plate.getSelected();
	
		if (wells.length != 2)
		{
			Forms.showNotification(event.currentTarget, 'Exactly 2 wells must be selected.');
			return;
		}
	
		var rna0 = wells[0].extract;
		var rna1 = wells[1].extract;
		
		wells[0].setExtract(rna1);
		wells[1].setExtract(rna0);
		Plate.paint(wells);
	}

	// Toggle the 'special select' menu on and off
	selectrna.toggleSpecialSelect = function(event)
	{
		var pos = Doc.getElementPosition('iconSpecialSelect');
		Menu.toggleTopMenu('menuSpecialSelect', pos.left+pos.width/4, pos.top+pos.height); 
		event.stopPropagation();
	}
	
	var contextRNA = null;
	var contextX;
	var contextY;
	/**
		Reacts to 'mouseup' and 'contextmenu' events for the bioplate. 
		This should bring up the cut/copy/paste context menu depending on which
		mouse button that was clicked.
	*/
	selectrna.contextEvent = function(event)
	{
		var frm = document.forms['reggie'];
		// Context menu on 'right' mouse button
		// Can't just check the button since two events are sent ('mouseup' and 'contextmenu')
		var showContext = event.type == 'contextmenu' && event.button == 2;
			
		if (showContext)
		{
			event.preventDefault(); // Prevents the default right-click menu from appearing
			
			// Get the well that is right-clicked and the RNA that is in it
			contextRNA = null;
			var well = event.target;
			while (well && (!well.id || well.id.indexOf('well') != 0))
			{
				well = well.parentNode;
			}
			if (well)
			{
				var c = well.id.split(/\./);
				contextRNA = Plate.getWell(parseInt(c[1]), parseInt(c[2])).extract;
			}
			// Update the context meny
			var caseSummaryMenu = Doc.element('mnuCaseSummary');
			if (contextRNA)
			{
				caseSummaryMenu.title = 'Show case summary for ' + Strings.encodeTags(contextRNA.name);
				Doc.show('sepCaseSummary');
				Doc.show('mnuCaseSummary');
			}
			else
			{
				Doc.hide('sepCaseSummary');
				Doc.hide('mnuCaseSummary');
			}
			
			var menu = Doc.element('menuContext');
			// 1 pixel offset to avoid losing well focus outline
			contextX = event.clientX+1;
			contextY = event.clientY+1;
			// Need short delay since 'mouseup' are also sent as 'click' events
			// to the 'document' object which BASE already have a Menu.hideAll()
			// call which would hide the menu immediately
			setTimeout(selectrna.showContextMenu, 100);
		}
	}
	
	selectrna.showContextMenu = function()
	{
		Menu.showTopMenu('menuContext', contextX, contextY);
	}

	selectrna.showCaseSummary = function()
	{
		if (!contextRNA) return;
		var caseName = contextRNA.name.substring(0, 7);
		var url = '../reports/case_summary.jsp?ID='+App.getSessionId();
		url += '&caseName='+encodeURIComponent(caseName);
		url += '&pageType=popup';
		Dialogs.openPopup(url, 'CaseSummary'+caseName, 1000, 700);
	}
	
	selectrna.showFlaggedRna = function()
	{
		currentSelected = Plate.getSelected();
		currentIndex = 0;

		var flagged = Rna.getFlagged();
		var url = 'show_flagged_rna.jsp?ID='+App.getSessionId();
		url += '&numFlagged='+flagged.length;
		url += '&numSelected='+currentSelected.length;
		Dialogs.openPopup(url, 'FlaggedRna', 800, 500);
	}

	
	//Flag the selected RNA
	selectrna.flagSelected = function(event)
	{
		var flag = Data.get(event.currentTarget, 'flag');
		var wells = Plate.getSelected();
		
		if (wells.length == 0)
		{
			Forms.showNotification(event.currentTarget, 'No wells have been selected.');
			return;
		}
		
		var comment = null;
		if (flag == 'ManualFlag')
		{
			// Check if a comment has been set on any of the RNA
			for (var i = 0; i < wells.length; i++)
			{
				var well = wells[i];
				if (well.extract && well.extract.info) 
				{
					if (well.extract.info.comment) comment = well.extract.info.comment;
				}
			}
			comment = prompt('Comment', comment || '');
			if (!comment) return;
		}
		
		var count = 0;
		for (var i = 0; i < wells.length; i++)
		{
			var well = wells[i];
			var rna = well.extract;
			if (rna && !rna.stratagene && !rna.external)
			{
				Rna.flag(rna, flag);
				rna.info.comment = comment;
				well.setExtract(null);
				count++;
			}
			well.selected = false;
		}
		
		selectrna.updateNumFlaggedRna();
		Plate.paint(wells);
	}


	selectrna.validateStep1 = function(event)
	{
		if (!quantitiesAreValid) 
		{
			event.preventDefault();
			return;
		}

		var frm = document.forms['reggie'];
		var numErrors = 0;
		var numWarnings = 0;
		var numRna = 0;
		var numStratagene = 0;
		var numExternal = 0;
		var wells = Plate.getWells();
		var schema = POOL_SCHEMA[frm.pool_schema.selectedIndex];
		for (var i = 0; i < wells.length; i++)
		{
			var well = wells[i];
			var rna = well.extract;
			if (well.hasWarning())
			{
				numWarnings++;
			}
			if (well.hasError())
			{
				numErrors++;
			}
			else
			{
				if (rna && rna.id) 
				{
					numRna++;
					if (rna.stratagene) numStratagene++;
					if (rna.external) numExternal++;
				}
			}
		}

		if (numErrors > 0)
		{
			event.preventDefault();
			return;
		}
		
		if (numRna == 0)
		{
			alert('There is no RNA in any wells');
			event.preventDefault();
			return;
		}
		
		if (numWarnings > 0)
		{
			if (!confirm('There are ' + numWarnings + ' wells with a warning. Continue anyway?'))
			{
				event.preventDefault();
				return;
			}
		}
		
		if (schema)
		{
			var numPrimary = schema.numPools * schema.numWellsPerPool;
			if (numRna < numPrimary)
			{
				if (!confirm((numPrimary-numRna)+ ' primary wells are missing RNA. Continue anyway?'))
				{
					event.preventDefault();
					return;
				}
			}
		}
		
		if (numRna == numStratagene+numExternal)
		{
			if (!confirm('There are only wells with Stratagene or external RNA on this plate. Continue anyway?'))
			{
				event.preventDefault();
				return;
			}
		}
		
	}
	
	selectrna.submit = function()
	{
		var frm = document.forms['reggie'];

		var submitInfo = {};
		var plateInfo = {};
		var flaggedRnaInfo = [];
		
		submitInfo.bioplate = plateInfo;
		submitInfo.flagged = flaggedRnaInfo;
		var schema = POOL_SCHEMA[frm.pool_schema.selectedIndex];
		
		plateInfo.name = Plate.name;
		plateInfo.comments = frm.comments.value;
		plateInfo.poolSchema = schema ? schema.id : null;
		plateInfo.wells = [];

		var wells = Plate.getWells();
		for (var i = 0; i < wells.length; i++)
		{
			var well = wells[i];
			var rna = well.extract;
			if (rna && rna.id)
			{
				var tmp = {};
				tmp.row = well.row;
				tmp.column = well.column;
				tmp.rna = {};
				tmp.rna.id = rna.id;
				tmp.rna.usedQuantity = rna.qc ? QUANTITY_QC : QUANTITY_REGULAR;
				tmp.rna.comment = rna.comment;
				plateInfo.wells[plateInfo.wells.length] = tmp;
			}
		}

		var flagged = Rna.getFlagged();
		for (var i = 0; i < flagged.length; i++)
		{
			var rna = flagged[i];
			var tmp = {};
			tmp.id = rna.id;
			tmp.flag = rna.flag;
			tmp.comment = rna.comment;
			flaggedRnaInfo[flaggedRnaInfo.length] = tmp;
		}

		var url = '../MRna.servlet?ID='+App.getSessionId();
		url += '&cmd=CreateMRnaPlate';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, selectrna.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	selectrna.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}
	
	selectrna.initElements = function(element, autoInit)
	{
		if (autoInit == 'plate-col')
		{
			Events.addEventHandler(element, 'click', selectrna.toggleColumn);
			Events.addEventHandler(element, 'mouseover', selectrna.highlightColumn);
			Events.addEventHandler(element, 'mouseout', selectrna.highlightColumn);
		}
		else if (autoInit == 'plate-row')
		{
			Events.addEventHandler(element, 'click', selectrna.toggleRow);
			Events.addEventHandler(element, 'mouseover', selectrna.highlightRow);
			Events.addEventHandler(element, 'mouseout', selectrna.highlightRow);
		}
		else if (autoInit == 'plate-well')
		{
			Events.addEventHandler(element, 'click', selectrna.toggleWell);
			Events.addEventHandler(element, 'mouseover', selectrna.highlightReplicated);
			Events.addEventHandler(element, 'mouseout', selectrna.highlightReplicated);
		}
		else if (autoInit == 'special-select')
		{
			Events.addEventHandler(element, 'click', selectrna.specialToggle)
		}
	}

	// Toggle the selected status of a single well
	selectrna.toggleWell = function(event)
	{
		var row = Data.int(event.currentTarget, 'row');
		var column = Data.int(event.currentTarget, 'col');
		var well = Plate.getWell(row, column);
		Plate.toggleSelected([well]);
	}

	// Toggle the selected status of a complete row
	selectrna.toggleRow = function(event)
	{
		var row = Data.int(event.currentTarget, 'row');
		Plate.toggleSelected(Plate.getRow(row));
	}

	// Toggle the selected status of a complete column
	selectrna.toggleColumn = function(event)
	{
		var column = Data.int(event.currentTarget, 'col');
		Plate.toggleSelected(Plate.getColumn(column));
	}
	
	// Toggle the selected status of a pool
	selectrna.togglePool = function(event)
	{
		var pool = Data.int(event.target, 'pool-num');
		if (!isNaN(pool)) Plate.toggleSelected(Plate.getPool(pool));
	}
	
	// Highlight enable/disable all wells in a column
	selectrna.highlightColumn = function(event)
	{
		var column = Data.int(event.currentTarget, 'col');
		var on = event.type == 'mouseover';
		
		Doc.addOrRemoveClass(event.currentTarget, 'highlight-column', on);
		var wells = Plate.getColumn(column);
		Plate.setHighlight(wells, 'highlight-column', on);
	}

	// Highlight enable/disable all wells in a row
	selectrna.highlightRow = function(event)
	{
		var row = Data.int(event.currentTarget, 'row');
		var on = event.type == 'mouseover';
		
		Doc.addOrRemoveClass(event.currentTarget, 'highlight-row', on);
		var wells = Plate.getRow(row);
		Plate.setHighlight(wells, 'highlight-row', on);
	}
	
	// Highligt enable/disable all wells in a pool
	selectrna.highlightPool = function(event)
	{
		var pool = Data.int(event.target, 'pool-num');
		var on = event.type == 'mouseover';
		if (!isNaN(pool))
		{
			Doc.addOrRemoveClass(event.target, 'highlight-pool', on);
			var wells = Plate.getPool(pool);
			Plate.setHighlight(wells, 'highlight-pool', on);
		}
	}

	/**
		Highlight all replicated wells with the same RNA as the given well.
	*/
	selectrna.highlightReplicated = function(event)
	{
		var column = Data.int(event.currentTarget, 'col');
		var row = Data.int(event.currentTarget, 'row');
		var on = event.type == 'mouseover';
		
		var well = Plate.getWell(row, column);
		if (well.extract && well.replicate)
		{
			// Get center coordinates for the current well
			var pos = Doc.getElementPosition(well.tag);
			var jsPos = new jsPoint(pos.left+pos.width/2, pos.top+pos.height/2);
	
			var replicated = Plate.getWellsByName(well.extract.name);
			for (var i = 0; i < replicated.length; i++)
			{
				var rep = replicated[i];
				if (rep != well)
				{
					Doc.addOrRemoveClass(rep.tag, 'highlight-replicated', on);
					if (rep.line)
					{
						// Clear any recent lines
						graphics.clearDrawing(rep.line);
						rep.line = null;
					}
					if (on)
					{
						// We draw a line between the current and replicated well
						var rPos = Doc.getElementPosition(rep.tag);
						rep.line = graphics.drawLine(pen, jsPos, new jsPoint(rPos.left+rPos.width/2, rPos.top+rPos.height/2));
					}
				}
			}
		}
	}

	// Some special toogle operations
	selectrna.specialToggle = function(event)
	{
		var what = Data.get(event.currentTarget, 'special');
		var wells = [];
		if (what == 'all' || what == 'empty' || what == 'none')
		{
			// All wells or all empty (will be filtered later)
			wells = Plate.getWells();
		}
		else if (what == 'pools' || what == 'empty-pools')
		{
			// All primary pools or all empty in the primary pools (will be filtered later)
			for (var i = 0; i < Plate.poolSchema.numPools; i++)
			{
				wells = wells.concat(Plate.getPool(i));
			}
		}
		else if (what == 'stratagene')
		{
			// All wells with 'Stratagene'
			var tmp = Plate.getWells();
			for (var i = 0; i < tmp.length; i++)
			{
				var well = tmp[i];
				if (well.extract && well.extract.stratagene) wells[wells.length] = well;
			}
		}
		else if (what == 'external')
		{
			// All wells with 'External RNA'
			var tmp = Plate.getWells();
			for (var i = 0; i < tmp.length; i++)
			{
				var well = tmp[i];
				if (well.extract && well.extract.external) wells[wells.length] = well;
			}
		}
		else if (what == 'replicates')
		{
			// All wells with replicated RNA
			var tmp = Plate.getWells();
			for (var i = 0; i < tmp.length; i++)
			{
				var well = tmp[i];
				if (well.extract && well.replicate) wells[wells.length] = well;
			}
		}
		else if (what == 'error')
		{
			// All wells with an error
			var tmp = Plate.getWells();
			for (var i = 0; i < tmp.length; i++)
			{
				var well = tmp[i];
				if (well.hasError()) wells[wells.length] = well;
			}
		}
		else if (what == 'warning')
		{
			// All wells with a warning
			var tmp = Plate.getWells();
			for (var i = 0; i < tmp.length; i++)
			{
				var well = tmp[i];
				if (well.warning) wells[wells.length] = well;
			}
		}
		
		// Extra filter for empty wells only
		if (what.indexOf('empty') != -1)
		{
			var tmp = wells;
			wells = [];
			for (var i = 0; i < tmp.length; i++)
			{
				if (!tmp[i].extract) wells[wells.length] = tmp[i];
			}
		}
		
		if (what == 'none')
		{
			Plate.setSelected(wells, false);
		}
		else
		{
			Plate.toggleSelected(wells);
		}
	}


	return selectrna;
}();

Doc.onLoad(SelectRna.initPage);
Doc.addElementInitializer(SelectRna.initElements);


var Rna = function()
{
	var rna = {};
	var flagged = [];
	var info = [];
	
	/**
		Create a new RNA object by name. More information
		about the RNA is automatically loaded from the database.
	*/
	rna.createByName = function(name)
	{
		var tmp = {};
		tmp.name = name;
		tmp.stratagene = Reggie.isStratagene(name)
		tmp.external = Reggie.isExternal(name);
		tmp.info = rna.infoByName(name);
		tmp.id = tmp.info.id;
		return tmp;
	}
	
	/**
		Create a new RNA object by info object.
	*/
	rna.createByInfo = function(info)
	{
		var tmp = {};
		tmp.name = info.name;
		tmp.stratagene = Reggie.isStratagene(tmp.name);
		tmp.external = Reggie.isExternal(tmp.name);
		tmp.id = info.id;
		tmp.info = info;
		return tmp;
	}
	
	/**
		Get information about a RNA item with a given name.
	*/
	rna.infoByName = function(name)
	{
		var key = 'N'+name;
		if (!info[key])
		{
			rna.loadInfoByNames([name]);
			if (!info[key]) info[key] = {};
		}
		return info[key];
	}

	/**
		Load and cache RNA information for all RNA items with a name in the
		given list.
	*/
	rna.loadInfoByNames = function(names)
	{
		var newNames = [];
		for (var i = 0; i < names.length; i++)
		{
			if (!info['N'+names[i]]) newNames[newNames.length] = names[i];
		}
		
		var submitInfo = {};
		submitInfo.names = newNames;
		
		if (newNames.length > 0)
		{
			var url = '../MRna.servlet?ID='+App.getSessionId();
			url += '&cmd=GetRnaInfoFromNames';	
			var response = Wizard.syncJsonRequest(url, 'POST', JSON.stringify(submitInfo));
			rna.cacheInfo(response.rna);
		}
	}
	
	rna.cacheInfo = function(rnaList)
	{
		for (var i = 0; i < rnaList.length; i++)
		{
			var r = rnaList[i];
			info['N'+r.name] = r;
			info['I'+r.id] = r;
			if (r.flag) flagged[flagged.length] = r;
		}
	}

	rna.unflag = function(r)
	{
		if (!r.info || !r.info.flag) return false;

		r.info.flag = null;
		for (var i = 0; i < flagged.length; i++)
		{
			if (r.id == flagged[i].id)
			{
				flagged.splice(i, 1);
				break;
			}
		}
		return true;
	}
	
	rna.flag = function(r, flag)
	{
		if (!r.info || r.info.flag) return false;
		r.info.flag = flag;
		flagged[flagged.length] = r.info;
	}
	
	rna.getFlagged = function()
	{
		return flagged;
	}
	
	return rna;
}();



var WellPainter = function()
{
	var painter = {};
	
	// Add class indicators for replicates and QC assigned wells
	painter.getClassNameForWell = function(well)
	{
		var cls = '';
		if (well.extract)
		{
			if (well.replicate) cls += ' replicate';
			if (well.extract.qc) cls += ' qc';
		}
		return cls;
	}
	
	painter.getWellText = function(well)
	{
		var text = '';
		if (well.duplicates)
		{
			well.setError('Duplicate RNA in this location');
			text += '<div class="name">'+Strings.encodeTags(well.duplicates.join(', '))+'</div>';
		}
		else if (well.extract)
		{
			// The well contains RNA
			var rna = well.extract;
			var info = rna.info;
			text += '<div class="name">'+Strings.encodeTags(rna.name)+'</div>';
			
			var warningMsg = [];
			if (info.bioWell)
			{
				var rnaWell = info.bioWell;
				text += '<div class="location">'+Strings.encodeTags(rnaWell.bioPlate.name+'['+rnaWell.location)+']</div>';
			}
			else if (!rna.stratagene && !rna.external)
			{
				warningMsg[warningMsg.length] = 'No location';
			}

			if (info && info.id && !rna.stratagene && !rna.external)
			{
				var quantity = 10000 * (rna.qc ? QUANTITY_QC : QUANTITY_REGULAR);
				var use = Math.ceil(quantity/info.NDConc) / 10;
				var water = Math.round(50-use);
				if (info.remainingQuantity)
				{
					text += '<div class="quantity">'+Numbers.formatNumber(info.remainingQuantity, 2) + 'µg</div>';
					// Must have at least 1.1µg or 1.22µg (if default values are used)
					var remainLimit = rna.qc ? QUANTITY_QC : QUANTITY_REGULAR;
					if (info.remainingQuantity < remainLimit)
					{
						warningMsg[warningMsg.length] = 'Low quantity';
					}
					// Warning if near the limit
					if (info.remainingQuantity < LOW_QUANTITY_WARNING_LIMIT)
					{
						warningMsg[warningMsg.length] = 'Low quantity';
					}
				}
				else
				{
					warningMsg[warningMsg.length] = 'No quantity';
				}
				if (info.rqs)
				{
					text += '<div class="quality-score">RQS='+Numbers.formatNumber(info.rqs, 1) + '</div>';
					if (info.rqs < QUALITY_SCORE_WARNING_LIMIT) warningMsg[warningMsg.length] = 'Low RQS value';
				}
				else if (info.rin)
				{
					text += '<div class="quality-score">RIN='+Numbers.formatNumber(info.rin, 1) + '</div>';
					if (info.rin < QUALITY_SCORE_WARNING_LIMIT) warningMsg[warningMsg.length] = 'Low RIN value';
				}
				else
				{
					warningMsg[warningMsg.length] = 'No RQS/RIN value';
				}
				if (info.NDConc)
				{
					text += '<div class="ndconc">'+Numbers.formatNumber(info.NDConc, 2) + 'ng/µl</div>';
					text += '<div class="volumes"><span class="volume">'+Numbers.formatNumber(use, 1)+'</span> + <span class="water">'+Numbers.formatNumber(water, 1)+'µl</span></div>';
				}
				else
				{
					warningMsg[warningMsg.length] = 'No NDConc value';
				}
				if (info.QiaCubeDate)
				{
					text += '<div class="qiacube-date">'+info.QiaCubeDate+'</div>';
				}
				else
				{
					warningMsg[warningMsg.length] = 'No QiaCubeDate value';
				}
			}
			else if (!rna.id)
			{
				well.setError('RNA not found');
			}
			if (rna.comment)
			{
				text += '<div class="comment">'+Strings.encodeTags(rna.comment)+'</div>';
			}
			if (info && info.comment)
			{
				text += '<div class="comment">'+Strings.encodeTags(info.comment)+'</div>';
			}
			if (warningMsg.length > 0)
			{
				well.setWarning(warningMsg.join('; '));
			}
		}
		else if (well.copyText)
		{
			text = '<div class="copy-text">'+well.copyText+'</div>';
		}
		else
		{
			text = 'empty';
		}
		return text;
	}
	
	return painter;
}();
